#include  <msp430x26x.h>
#include  <stdint.h>

#define   Num_Samples   16

volatile uint16_t A2results[Num_Samples];  
volatile uint16_t A3results[Num_Samples];  

void main(void)
{
  WDTCTL = WDTPW+WDTHOLD;                   // Stop watchdog timer
  P6SEL = BIT2 + BIT3;                      // Enable A/D channel inputs
  
  P6DIR = BIT4;  P6OUT = BIT4;
  P5DIR = BIT5;  P5OUT = BIT5;
  P2DIR = BIT0;  P2OUT = BIT0;
  
  /* Configure TimerB to produce the triggers for the ADC */
  TBCTL = TBSSEL_1 | MC_1 | TBCLR;              // Use ACLK, count up, reset counter and direction
  TBCCR0 = 32768;                               // Init TBCCR0 w/ sample period
  TBCCR1 = 32738;                               // Trigger for ADC12 Sample Conversion
  TBCCTL1 = OUTMOD_7;                           // Reset OUT1 on EQU1, set on EQU0
  
  /* Configure ADC for repeated sampling of A2 and A3 */
  ADC12CTL0 = ADC12ON | MSC | SHT0_8;           // Turn on ADC12, extend sampling time to avoid overflow of results
  ADC12CTL1 = SHP + SHS_3 | CONSEQ_3 | ADC12SSEL_1;  // Use ACLK timer, repeated sequence
  
  ADC12MCTL0 = SREF_2 | INCH_2;                 // ref+=VeREF+, channel = A2
  ADC12MCTL1 = SREF_2 | INCH_3 | EOS;           // ref+=VeREF+, channel = A3
  
  ADC12IFG = 0;
  ADC12CTL0 |= ENC;                             // Enable conversions

  /* Configure DMA to transfer A2 and A3 values from the ADC to RAM */
  DMACTL0 = DMA0TSEL_6 + DMA1TSEL_6;            // ADC12IFG0 triggers DMA0, ADC12IFG1 triggers DMA1
  
  DMA0SA = ADC12MEM0_;                          // Source address
  DMA0DA = (unsigned short) &A2results[0];      // Destination address
  DMA0SZ = 16;                                  // Transfer Num_Samples words
  DMA0CTL = DMADT_4 | DMADSTINCR_3 | DMAEN;     // Repeated block transfer, increment dest addr, enable DMA
 
  DMA1SA = ADC12MEM1_;                          // Source
  DMA1DA = (unsigned short) &A3results[0];      // Destination
  DMA1SZ = 16;                                  // Transfer Num_Samples words
  DMA1CTL = DMADT_4 | DMADSTINCR_3 | DMAEN | DMAIE;  // Repeated block transfer, increment dest addr, enable DMA & interrupt
  
  _BIS_SR(GIE);                                 // Enable interrupts
  
  while(1);
}

/* DMA interrupt, triggers at the end of DMA1's 16th sample transfer */
#pragma vector=DMA_VECTOR
__interrupt void DMA0_ISR (void)
{
    switch(__even_in_range(DMAIV,16))
    {
        case 0: break;  // No interrupt
        case 2:         // DMA0IFG
            _NOP();     // Shouldn't be here
            break;
        case 4:         // DMA1IFG
            _NOP();     // Change double buffer
            break;
        case 6: break;  // DMA2IFG
        case 8: break;  // Reserved
        case 10: break; // Reserved
        case 12: break; // Reserved
        case 14: break; // Reserved
        case 16: break; // Reserved
        default: break;
    }
}